<?php
// $Id: myhookadmin.inc,v 1.3 2010/08/23 05:01:59 ufku Exp $

/**
 * Generates admin page.
 */
function myhookadmin_admin() {
  $output = array();
  $output['form'] = drupal_get_form('myhookadmin_admin_form');
  $output['tips'] = array('#markup' => '<h2>Tips & Links</h2>
  <ul class="tips">
    <li>Example hook_node_insert() implementation that notifies site admin of new content.<br />
    <code>function myhook_node_insert($node) {<br />
&nbsp;  drupal_mail_system(\'myhook\', \'mail\')->mail(array(<br />
&nbsp; &nbsp; \'to\' => \'admin@example.com\',<br />
&nbsp; &nbsp; \'subject\' => \'New content\',<br />
&nbsp; &nbsp; \'body\' => \'New content entered at \' . url(\'node/\' . $node->nid, array(\'absolute\' => TRUE)),<br />
&nbsp; &nbsp; \'headers\' => array(\'From\' => variable_get(\'site_mail\', ini_get(\'sendmail_from\'))),<br />
&nbsp; ));<br />
}</code>
    </li>
    <li>If your code causes a fatal error or if you want to disable it for a single request, just append <code>?myhook=0</code> to the URL. Ex: ' . url('admin/config/development/myhook', array('query' => array('myhook' => '0'), 'absolute' => TRUE)) . '</li>
    <li>You may need to flush caches to make some hooks such as <code>hook_menu()</code> or <code>hook_theme()</code> to take effect.</li>
    <li><a href="http://api.drupal.org/api/group/hooks/7">Drupal core hooks</a></li>
    <li><a href="http://api.lullabot.com/group/views_hooks/7">Views hooks</a></li>
    <li><a href="http://api.lullabot.com/group/hooks/7">Various module hooks</a></li>
  </ul>',
  );
  return $output;
}

/**
 * Returns admin form.
 */
function myhookadmin_admin_form($form, &$form_state) {
  $form['code'] = array(
    '#title' => 'Custom code',
    '#type' => 'textarea',
    '#default_value' => myhookadmin_myhook_code(),
    '#rows' => 20,
    '#cols' => 60,
    '#description' => 'Define your custom hooks as if they are part of myhook.module. Do not include PHP opening and closing tags.',
  );
  $form['is_inc'] = array(
    '#title' => 'Save the code in a file and include it as a file',
    '#type' => 'checkbox',
    '#default_value' => variable_get('myhookadmin_is_inc', 0),
    '#description' => 'This may save a DB hit and an eval execution on each request as the code will be saved in a file whose path must be defined below.',
  );
  $form['inc_path'] = array(
    '#title' => 'File path to include',
    '#type' => 'textfield',
    '#size' => 60,
    '#default_value' => variable_get('myhookadmin_inc_path', ''),
    '#description' => 'This must be a PHP-writable file path such as ' . file_directory_path('public') . '/myhook.inc. It will be made readonly automatically after writing.',
  );
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => 'Save',
  );
  $form['#validate'][] = 'myhookadmin_admin_validate';
  $form['#submit'][] = 'myhookadmin_admin_submit';
  return $form;
}

/**
 * Validates admin form.
 */
function myhookadmin_admin_validate($form, &$form_state) {
  $is_inc = $form_state['values']['is_inc'];
  $inc_path = $form_state['values']['inc_path'];
  // Check if file is writable
  if ($is_inc) {
    if (empty($inc_path)) {
      return form_set_error('inc_path', 'Please specify a valid file path.');
    }
    if (file_exists($inc_path)) {
      @drupal_chmod($inc_path, 0666);
    }
    if (!$fp = fopen($inc_path, 'w')) {
      return form_set_error('inc_path', 'Unable to open the specified file for writing.');
    }
    fwrite($fp, "<?php\n\n" . $form_state['values']['code']);
    fclose($fp);
    @drupal_chmod($inc_path, 0444);
  }
}

/**
 * Submits admin form.
 */
function myhookadmin_admin_submit($form, &$form_state) {
  $is_inc = $form_state['values']['is_inc'];
  $inc_path = $form_state['values']['inc_path'];
  $code = $form_state['values']['code'];

  // Save code
  db_update('myhookadmin')
  ->fields(array('value' => $code))
  ->condition('name', 'myhook_code')
  ->execute();

  // Save variables
  variable_set('myhookadmin_is_inc', $is_inc);
  variable_set('myhookadmin_inc_path', $inc_path);

  drupal_set_message('The changes have been saved.');
}